//
//  MRSharingSession.h
//  Meridian
//
//  Copyright © 2016 Aruba Networks. All rights reserved.
//

#import <Foundation/Foundation.h>
@class MREditorRequestManager;
@protocol MRSharingSessionDelegate;

/// Represents an authenticated user. Used to authorize requests on the `MRFriend` and `MRInvite`.
/// Additionally will post location changes and listen for updates in friend locations.
@interface MRSharingSession : NSObject
NS_ASSUME_NONNULL_BEGIN

/// Creates an account with the given password and user details, and returns an MRSharingSession upon
/// success.
///
/// @param password      The password to create the account with
/// @param name          User's name
/// @param appKey        App key the user belongs to
/// @param completion    Block to be called upon completion with a MRSharingSession or an error
+ (void)createAccountWithPassword:(NSString *)password name:(NSString *)name appKey:(MREditorKey *)appKey completion:(void (^_Nullable)(MRSharingSession *_Nullable session, NSError *_Nullable error))completion;

/// Logs into an account with the given password and userKey, and returns an MRSharingSession upon
/// success.
///
/// @param key           User's identifier
/// @param password      User's password
/// @param appKey        App key the user belongs to
/// @param completion    Block to be called upon completion with a MRSharingSession or an error
+ (void)loginWithKey:(MREditorKey *)key password:(NSString *)password appKey:(MREditorKey *)appKey completion:(void (^_Nullable )(MRSharingSession *_Nullable session, NSError *_Nullable error))completion;

/// Permanently deletes an account. The session will be invalid upon completion.
///
/// @param completion   Block to be called upon completion with an error upon failure
- (NSOperation *)deleteAccountWithCompletion:(void (^_Nullable )(NSError *_Nullable error))completion;

/// Begins listening for updates in friends until `stopListeningForUpdates` is called. Changes will
/// call the delegate's `friendSession:friendsDidUpdate:` method.
- (void)startListeningForUpdates;

/// Stops listening for updates in friends.
- (void)stopListeningForUpdates;

/// Begins posting location updates until `stopPostingLocationUpdates` is called.
- (void)startPostingLocationUpdates;

/// Stops posting location updates.
- (void)stopPostingLocationUpdates;

/// The delegate for this session.
@property (nonatomic, weak, nullable) id<MRSharingSessionDelegate> delegate;

/// The authenticated user's identifier.
@property (nonatomic, readonly) MREditorKey *key;

/// The appKey for this session.
@property (nonatomic, readonly) MREditorKey *appKey;

NS_ASSUME_NONNULL_END
@end

/// Delegate methods for `MRSharingSession`.
@protocol MRSharingSessionDelegate <NSObject>
NS_ASSUME_NONNULL_BEGIN
@optional

/// Called when the user's shared items have updated.
///
/// @param session      The session that encountered the failure
/// @param friends      The user's friends as an array of MRFriend
- (void)sharingSession:(MRSharingSession *)session friendsDidUpdate:(NSArray <MRFriend *> *)friends;

/// Called when the friend session encounters an error.
///
/// @param session      The session that encountered the failure
/// @param error        The error that was generated by the failure
- (void)sharingSession:(MRSharingSession *)session didFailWithError:(NSError *)error;

NS_ASSUME_NONNULL_END
@end
