//
//  MSSearchViewController.m
//  MeridianSamples
//
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

#import "MSSearchViewController.h"

@interface MSSearchViewController ()
@property (nonatomic, copy) MREditorKey *appKey;
@property (nonatomic, strong) UISearchBar *searchBar;
@property (nonatomic, strong) MRSearch *search;
@property (nonatomic, strong) NSTimer *autoUpdateTimer;
@property (nonatomic, strong) NSMutableArray *searchResults;
@end

@implementation MSSearchViewController

- (instancetype)init {
    if ((self = [super initWithStyle:UITableViewStylePlain])) {
        self.edgesForExtendedLayout = UIRectEdgeNone;
    }
    return self;
}

- (void)loadView {
    [super loadView];
    
    self.appKey = [MREditorKey keyWithIdentifier:APP_ID];
    
    self.searchBar = [[UISearchBar alloc] initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, 44)];
    self.searchBar.delegate = self;
    self.searchBar.placeholder = @"Find placemarks, etc";
    self.navigationItem.titleView = self.searchBar;
}

- (void)startSearch {
    if (self.searchBar.text.length) {
        MRSearchRequest *searchRequest = [MRSearchRequest new];
        searchRequest.app = self.appKey;
        searchRequest.naturalLanguageQuery = self.searchBar.text;
        
        self.search = [[MRSearch alloc] initWithRequest:searchRequest];
        self.searchResults = [NSMutableArray array];
        [self.search startWithCompletionHandler:^(NSArray * _Nullable results, NSError * _Nullable error) {
            if (results) {
                [self.searchResults addObjectsFromArray:results];
                [self.tableView reloadData];
            } else if (error) {
                NSLog(@"Error loading search results: %@", error.localizedDescription);
            }
        }];
    } else {
        [self.search cancel];
        [self.searchResults removeAllObjects];
        [self.tableView reloadData];
    }
}

- (void)cancelTapped {
    self.searchBar.text = nil;
    [self.searchBar resignFirstResponder];
    [self startSearch];
}


#pragma mark - MSExampleViewController

+ (NSString *)exampleTitle {
    return @"Search";
}

+ (NSString *)exampleInfo {
    return @"Example of how to use the MRSearch API to search for Placemarks and Tags and display the results.";
}


#pragma mark - UISearchBarDelegate

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar {
    UIBarButtonItem *cancelItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(cancelTapped)];
    [self.navigationItem setRightBarButtonItem:cancelItem animated:YES];
    
    return YES;
}

- (BOOL)searchBarShouldEndEditing:(UISearchBar *)searchBar {
    [self.navigationItem setRightBarButtonItem:nil animated:YES];
    
    return YES;
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    [self startSearch];
}

- (void)searchBar:(UISearchBar *)theSearchBar textDidChange:(NSString *)searchText {
    if (self.autoUpdateTimer) {
        [self.autoUpdateTimer invalidate];
        self.autoUpdateTimer = nil;
    }
    
    // automatically update search results 1 second after you finish typing, even if you didn't hit the Search button
    self.autoUpdateTimer = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(startSearch) userInfo:nil repeats:NO];
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView {
    [self.searchBar resignFirstResponder];
}


#pragma mark - UITableView

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.searchResults.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *reuseIdentifier = @"MRSearchResultCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
    if (cell == nil)
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:reuseIdentifier];
    
    // results will be instances of one of several possible classes
    id result = self.searchResults[indexPath.row];
    NSString *name, *kind;
    if ([result isKindOfClass:[MRPlacemark class]]) {
        name = ((MRPlacemark *)result).name ?: ((MRPlacemark *)result).typeName;
        kind = @"Placemark";
    } else if ([result isKindOfClass:[MRTag class]]) {
        name = ((MRTag *)result).name;
        kind = @"Tag";
    } else if ([result isKindOfClass:[MRMap class]]) {
        name = ((MRMap *)result).name;
        kind = @"Map";
    } else {
        name = @"Unknown name";
        kind = @"Unknown kind";
    }
    
    cell.textLabel.text = name;
    cell.detailTextLabel.text = kind;
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    id result = self.searchResults[indexPath.row];

    if ([result isKindOfClass:[MRPlacemark class]]) {
        [self presentDirectionsToPlacemark:result];
    } else if ([result isKindOfClass:[MRTag class]]) {
        [self loadLocationAndPresentTag: ((MRTag *)result).mac];
    } else if ([result isKindOfClass:[MRMap class]]) {
        // load map
        MRMapViewController *controller = [[MRMapViewController alloc] initWithNibName:nil bundle:nil];
        controller.mapView.mapKey = [MREditorKey keyForMap:((MRMap *)result).key.identifier app:self.appKey.identifier];
        [self.navigationController pushViewController:controller animated:YES];
    }
}

- (void)loadLocationAndPresentTag:(NSString *)tagMac {
    // MRTag returned via search does not contain location data, so use the asset-beacons API to get more info
    // Future SDK enhancements should make this explicit URL creation unnecessary
    
    NSString *tagEndpoint = [NSString stringWithFormat:@"https://edit.meridianapps.com/api/locations/%@/asset-beacons/%@", APP_ID, tagMac];
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] init];
    [request setHTTPMethod:@"GET"];
    [request setURL:[NSURL URLWithString:tagEndpoint]];
    
    NSURLSession *session = [NSURLSession sessionWithConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [[session dataTaskWithRequest:request completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        if (error) {
            NSLog(@"Request error %@", error.localizedDescription);
            return;
        }
      
        NSInteger statusCode = ((NSHTTPURLResponse *)response).statusCode;
        if (statusCode != 200) {
            NSLog(@"Error: HTTP status code %li", (long)statusCode);
            return;
        }
        
        if (!data) {
            NSLog(@"Error: did not receive tag data");
            return;
        }
        
        // Pull up the last map that tag was seen on
        NSDictionary *json = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
        NSString *mapID = json[@"calculations"][@"default"][@"location"][@"map_id"];
        if (!mapID) {
            NSLog(@"Unable to retrieve location info from tag response");
            return;
        }
        
        MRMapViewController *controller = [[MRMapViewController alloc] initWithNibName:nil bundle:nil];
        controller.mapView.mapKey = [MREditorKey keyForMap:mapID app:self.appKey.identifier];
        controller.mapView.showsTags = YES;
        controller.mapView.tagPredicate = [NSPredicate predicateWithFormat:@"mac LIKE %@", tagMac];

        dispatch_async(dispatch_get_main_queue(), ^{
            [self.navigationController pushViewController:controller animated:YES];
        });
    }] resume];
}

@end
