//
//  MSCampaignsViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit
import UserNotifications

class MSCampaignsTableViewController: UITableViewController, MRCampaignManagerDelegate, MSExampleViewController {
    private lazy var campaignManager: MRCampaignManager = {
        let campaignManager = MRCampaignManager.init(app: MREditorKey(identifier: APP_ID))
        campaignManager.delegate = self
        return campaignManager
    }()

    // We need to remember whether we were monitoring for campaigns previously in case we were relaunched - that way we can
    // resume monitoring which has the side-effect of triggering any campaigns that were waiting to be triggered on relaunch.
    private let MSCampaignsViewControllerShouldBeMonitoringKey = "MSCampaignsViewControllerShouldBeMonitoring"
    private var shouldBeMonitoring: Bool {
        get {
            return UserDefaults.standard.bool(forKey: MSCampaignsViewControllerShouldBeMonitoringKey)
        }
        set (newValue) {
            UserDefaults.standard.set(newValue, forKey: MSCampaignsViewControllerShouldBeMonitoringKey)
        }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        self.tableView.sectionFooterHeight = UITableViewAutomaticDimension
        self.tableView.estimatedSectionFooterHeight = 70

        // sign up to know when the application goes to foreground so we can clear any notifications from Notification Center
        NotificationCenter.default.addObserver(self, selector: #selector(applicationWillEnterForeground), name: NSNotification.Name.UIApplicationWillEnterForeground, object: nil)

        // resume monitoring if necessary
        if self.shouldBeMonitoring {
            self.campaignManager.startMonitoring()
        }
    }

    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        // let the user know that leaving while monitoring will cancel monitoring.
        if self.isMovingToParentViewController && self.shouldBeMonitoring {
            let alertController = UIAlertController(title: "Campaign Monitoring", message: "Campaign Monitoring has been stopped.", preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: "OK", style: .cancel)
            alertController.addAction(cancelAction)
            self.present(alertController, animated: true)

            // shut down our campaign manager if it was running
            self.campaignManager.stopMonitoring()
            self.shouldBeMonitoring = false

            // unregister for notifications
            NotificationCenter.default.removeObserver(self)
        }
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Campaigns"
    }

    static func exampleDetail() -> String {
        return "Example of how to use the MRCampaignManager API to listen for proximity beacons and trigger campaigns, as directed by the Meridian Editor."
    }

    // MARK: - UITableViewController

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = UITableViewCell.init(style: .default, reuseIdentifier: nil)
        cell.textLabel?.font = UIFont.systemFont(ofSize: 20)
        if indexPath.section == 0 {
            cell.textLabel?.text = self.shouldBeMonitoring ? "Stop Monitoring" : "Start Monitoring"
        } else {
            cell.textLabel?.text = "Reset Campaigns"
        }
        return cell
    }

    private let footerText = [
        "Starts or stops background monitoring for Campaigns. When started, Campaigns will be monitored in the background.\n\nIMPORTANT: Campaign triggers will only occur when this page is being displayed. You can leave the app or turn your screen off, but if the app is shut down, you will need to navigate back to this screen for monitoring to continue.",
        "Resets the Campaign 'cooldown' so that Campaign triggers can create notifications immediately instead of waiting for a set period (normally around 12 hours)"
    ]

    override func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        let footerView = UITableViewHeaderFooterView()
        if section < footerText.count {
            let label = createFooterLabel(footerText[section])
            label.autoresizingMask = [.flexibleHeight]
            footerView.addSubview(label)
        }
        return footerView
    }

    override func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        let label: UILabel = createFooterLabel(footerText[section])
        let size = label.sizeThatFits(CGSize(width: view.frame.size.width, height: CGFloat.greatestFiniteMagnitude))
        let padding: CGFloat = 20.0
        return size.height + padding
    }

    private func createFooterLabel(_ text: String) -> UILabel {
        let widthPadding: CGFloat = 20.0
        let label: UILabel = UILabel(frame: CGRect(x: widthPadding, y: 0, width: self.view.frame.size.width - widthPadding, height: 0))
        label.text = text
        label.numberOfLines = 0
        label.textAlignment = NSTextAlignment.left
        label.lineBreakMode = NSLineBreakMode.byWordWrapping
        label.font = UIFont.systemFont(ofSize: 13)
        return label
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if indexPath.section == 0 {
            if !self.shouldBeMonitoring {
                #if arch(i386) || arch(x86_64)
                    let alertController = UIAlertController(title: "Error", message: "Campaign Monitoring is not supported in the simulator.", preferredStyle: .alert)
                    let cancelAction = UIAlertAction(title: "OK", style: .cancel)
                    alertController.addAction(cancelAction)
                    self.present(alertController, animated: true)
                #else
                    // register to display local notifications
                    UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .badge, .sound]) { [weak self] (_, error) in
                        guard let strongSelf = self else {
                            return
                        }
                        if let requestError = error {
                            let alertController = UIAlertController(title: "Unauthorized for local notifications", message: requestError.localizedDescription, preferredStyle: .alert)
                            let cancelAction = UIAlertAction(title: "OK", style: .cancel)
                            alertController.addAction(cancelAction)
                            strongSelf.present(alertController, animated: true)
                            return
                        }
                        strongSelf.shouldBeMonitoring = true
                        strongSelf.campaignManager.startMonitoring()
                        DispatchQueue.main.async {
                            strongSelf.tableView.reloadData()
                        }
                    }
                #endif
            } else {
                self.campaignManager.stopMonitoring()
                self.shouldBeMonitoring = false
                self.tableView.reloadData()
            }
        } else if indexPath.section == 0 {
            self.campaignManager.resetAllCampaigns(completionHandler: { [weak self] error in
                if let campaignError = error {
                    let alertController = UIAlertController(title: "Campaign Reset Error", message: campaignError.localizedDescription, preferredStyle: .alert)
                    let cancelAction = UIAlertAction(title: "OK", style: .cancel)
                    alertController.addAction(cancelAction)
                    self?.present(alertController, animated: true)
                } else {
                    let alertController = UIAlertController(title: "Campaign Reset Complete", message: nil, preferredStyle: .alert)
                    let cancelAction = UIAlertAction(title: "OK", style: .cancel)
                    alertController.addAction(cancelAction)
                    self?.present(alertController, animated: true)
                }
            })
        }
        self.tableView.deselectRow(at: indexPath, animated: true)
    }

    // MARK: - MRCampaignManagerDelegate

    func campaignManager(_ manager: MRCampaignManager, shouldGenerateNotificationForCampaign campaignInfo: [AnyHashable : Any]) -> Bool {
        print("About to trigger campaign with title \(campaignInfo["title"] as? String ?? "No Title")")
        // We could return false or create our own UIAlert or NSNotification based on the contents of campaignInfo if we wished.
        // Returning true will cause the default behavior, typically a local notification.

        // Since we don't want to pollute our app delegate with logic to generate an alert if we're foregrounded, we'll handle
        // that case here.
        if UIApplication.shared.applicationState == .active {
            let alertController = UIAlertController(title: "Campaign Triggered", message: campaignInfo["message"] as? String, preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: "OK", style: .cancel)
            alertController.addAction(cancelAction)
            self.present(alertController, animated: true)

            // we handled it so MRCampaignManager doesn't have to.
            return false
        } else {
            // allow MRCampaignManager to handle it the usual way (posting a local notification)
            return true
        }
    }

    func campaignManager(_ manager: MRCampaignManager, didFailWithError error: Error) {
        print("Campaign manager failed with error: " + error.localizedDescription)
        let alertController = UIAlertController(title: "Campaign Error", message: error.localizedDescription, preferredStyle: .alert)
        let cancelAction = UIAlertAction(title: "OK", style: .cancel)
        alertController.addAction(cancelAction)
        self.present(alertController, animated: true)
    }

    // MARK: - Actions

    @objc private func applicationWillEnterForeground() {
        // clear out any notifications generated by MRCampaignManager
        UNUserNotificationCenter.current().removeAllPendingNotificationRequests()
    }
}
