//
//  MSDelegateViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/18/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSDelegateViewController: UIViewController, MRMapViewDelegate, MSExampleViewController {
    private lazy var mapView: MRMapView = {
        let mapView = MRMapView()
        mapView.delegate = self
        mapView.mapKey = MREditorKey(forMap:MAP_ID, app:APP_ID)
        return mapView
    }()
    var directions: MRDirections?

    override func viewDidLoad() {
        super.viewDidLoad()
        self.view = self.mapView

        let button0 = UIBarButtonItem(title: "Directions", style: .plain, target: self, action: #selector(self.directionsAction))
        self.navigationItem.rightBarButtonItems = [button0]
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Map Delegates"
    }

    static func exampleDetail() -> String {
        return "Map that logs delegate callbacks.\n\n" +
        "• All MRMapViewDelegate methods are logged to console.\n" +
        "• A button to start and stop directions.\n"
    }

    // MARK: - MRMapViewDelegate

    func mapViewWillStartLoadingMap(_ mapView: MRMapView) {
        print(#function)
    }

    func mapViewDidFinishLoadingMap(_ mapView: MRMapView) {
        print(#function)
    }

    func mapViewDidFailLoadingMap(_ mapView: MRMapView, withError error: Error?) {
        print(#function + " - error: \(error?.localizedDescription ?? "None")")
    }

    func mapViewDidChangeUseAccessiblePaths(_ mapView: MRMapView) {
        print(#function)
    }

    func mapView(_ mapView: MRMapView, willScrollToStepAt index: UInt) {
        print(#function + " - index: \(index)")
    }

    func mapView(_ mapView: MRMapView, visibleMapRectDidChange animated: Bool) {
        print(#function + " - animated: \(animated)")
    }

    func mapView(_ mapView: MRMapView, viewFor annotation: MRAnnotation?) -> MRAnnotationView? {
        print(#function + " - annotation: \(annotation?.description ?? "None")")

        if let placemark = annotation as? MRPlacemark {
            let defaultView = mapView.defaultView(for: placemark)
            defaultView.canTapCallout = true
            return defaultView
        }

        return nil
    }

    func mapView(_ mapView: MRMapView, didAdd views: [MRAnnotationView]) {
        print(#function + " - views: \(views.count)")
    }

    func mapView(_ mapView: MRMapView, didLoad placemarks: [MRPlacemark]) {
        print(#function + " - placemarks: \(placemarks.count)")
    }

    func mapView(_ mapView: MRMapView, didSelect view: MRAnnotationView) {
        print(#function + " - view: \(view.description)")
    }

    func mapView(_ mapView: MRMapView, didDeselect view: MRAnnotationView) {
        print(#function + " - view: \(view.description)")

    }

    func mapView(_ mapView: MRMapView, didTapCalloutFor view: MRAnnotationView) {
        print(#function + " - view: \(view.description)")
    }

    // MARK: - Action

    @objc private func directionsAction() {
        if self.mapView.route == nil && self.mapView.placemarks.count > 1 {
            guard let appKey = self.mapView.mapKey.parent else {
                return
            }
            guard let destinationPlacemark = self.mapView.placemarks[0] as? MRPlacemark else {
                return
            }
            guard let sourcePlacemark = self.mapView.placemarks[1] as? MRPlacemark else {
                return
            }
            let request = MRDirectionsRequest()
            request.app = appKey
            request.destination = MRDirectionsDestination(placemarkKey: destinationPlacemark.key)
            request.source = MRDirectionsSource(placemarkKey: sourcePlacemark.key)

            self.directions = MRDirections(request: request, presenting: self)
            self.directions?.calculate(completionHandler: {[weak self] (response, error) in
                self?.directionsResponseDidLoad(response: response, error: error)
            })
        } else {
            self.mapView.route = nil
        }
    }

    // MARK: - Internal

    private func directionsResponseDidLoad(response: MRDirectionsResponse?, error: Error?) {
        self.directions = nil
        if let routeResponse = response?.routes, error == nil, routeResponse.count > 0 {
            self.mapView.deselectAnnotation(animated: false)
            self.mapView.setRoute(routeResponse[0], animated: true)
            return
        }

        // Error or no routes:
        let alertController = UIAlertController(title: "Could not load directions", message: error?.localizedDescription, preferredStyle: .alert)
        let OKAction = UIAlertAction(title: "OK", style: .default)
        alertController.addAction(OKAction)
        self.present(alertController, animated: true)
    }
}
